-- Create table
CREATE TABLE IF NOT EXISTS `{table}`
	(`player_uuid` CHAR(36) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
	`key` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
	`value` DECIMAL(20,5) NOT NULL,
	UNIQUE `player_uuid_index` (`player_uuid`, `key`) USING BTREE,
	INDEX `ranking_index` (`key`, `value`) USING HASH);

-- Create stored procedure used to loop through passed stats
DROP procedure IF EXISTS `mbedwars_player_stats_update`;

CREATE PROCEDURE `mbedwars_player_stats_update`
	(_player_uuid CHAR(36) CHARACTER SET ascii COLLATE ascii_bin,
	_raw_stats MEDIUMTEXT)
BEGIN
	-- init everything
    DECLARE _next TEXT DEFAULT NULL;
    DECLARE _nextlen INT DEFAULT NULL;
    DECLARE _middlei INT DEFAULT NULL;
    DECLARE _key TEXT DEFAULT NULL;
    DECLARE _value DECIMAL(20,10) DEFAULT NULL;
    DECLARE _index INT DEFAULT 0;
    DECLARE _length INT DEFAULT 0;
    
    -- first go through all stats
    iterator:
    LOOP
    	-- exit the loop if the list seems empty or was null;
        -- this extra caution is necessary to avoid an endless loop in the proc.
        IF LENGTH(TRIM(_raw_stats)) = 0 OR _raw_stats IS NULL THEN
        	LEAVE iterator;
        END IF;
        
        -- capture the next value from the list
        SET _next = SUBSTRING_INDEX(_raw_stats,',',1);
        
        -- save the length of the captured value; we will need to remove this
        -- many characters + 1 from the beginning of the string 
        -- before the next iteration
        SET _nextlen = LENGTH(_next);
        
        -- parse the entry
        SET _key = SUBSTRING_INDEX(_next, '!', 1);
        SET _value = CAST(INSERT(_next, 1, LENGTH(_key)+1, '') AS DECIMAL(20, 10));
        
        REPLACE INTO `{table}` (`player_uuid`, `key`, `value`) VALUES (_player_uuid, _key, _value);
        
        -- rewrite the original string using the `INSERT()` string function,
        -- args are original string, start position, how many characters to remove, 
        -- and what to "insert" in their place (in this case, we "insert"
        -- an empty string, which removes _nextlen + 1 characters)
        SET _raw_stats = INSERT(_raw_stats, 1, _nextlen + 1,'');
    END LOOP;
END;

-- Create stored procedure used to gets stats from a player with their uuid
DROP procedure IF EXISTS `mbedwars_player_stats_get_by_uuid`;

CREATE PROCEDURE `mbedwars_player_stats_get_by_uuid`
	(_player_uuid CHAR(36) CHARACTER SET ascii COLLATE ascii_bin,
	_sort_stat_id VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin)
BEGIN
	SET @rank = 0;
	
	SELECT *, (SELECT rank
		FROM
		  (SELECT `player_uuid`,
		         @rank := IFNULL(@rank,0)+1 AS `rank`
		  FROM `{table}`
          WHERE `key` = _sort_stat_id
		  ORDER BY `value` DESC) AS T
		WHERE T.`player_uuid` = _player_uuid) AS `rank`
	FROM `{table}`
	WHERE `player_uuid` = _player_uuid;
END;

-- Create stored procedure used to gets stats from a player with their name
DROP procedure IF EXISTS `mbedwars_player_stats_get_by_name`;

CREATE PROCEDURE `mbedwars_player_stats_get_by_name`
	(_username VARCHAR(16) CHARACTER SET ascii COLLATE ascii_bin,
	_sort_stat_id VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin,
	_prop_username_id VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin,
	_prop_last_login_id VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin)
BEGIN
	SET @rank = 0;
	
	-- get uuid by name
	-- will also sort by last login when there are multiple entries with the same name
	SET @uuid = (SELECT tFinal.`player_uuid` FROM (SELECT tName.`player_uuid`, tLastLogin.`value` as `last_login`
	    FROM `{table_properties}` tName
        LEFT JOIN `{table_properties}` tLastLogin
        	ON tLastLogin.`id` = _prop_last_login_id
            	AND tLastLogin.`player_uuid` = tName.`player_uuid`
	    WHERE tName.`id` = _prop_username_id
	        AND tName.`value` = _username
        ORDER BY `last_login` DESC
        LIMIT 1) tFinal);
	        
	SELECT *, (SELECT rank
		FROM
		  (SELECT `player_uuid`,
		         @rank := IFNULL(@rank,0)+1 AS `rank`
		  FROM `{table}`
          WHERE `key` = _sort_stat_id
		  ORDER BY `value` DESC) AS T
		WHERE T.`player_uuid` = @uuid) AS `rank`
	FROM `{table}`
	WHERE `player_uuid` = @uuid;
END;